<?php
/*

Plugin Name: Dailymile Widgets
Plugin URI: http://www.dailymile.com/widgets
Description: Faster, cached Dailymile widgets.
Author: Joe Maller
Version: 1.0
Author URI: http://www.dailymile.com

*/


/**
 * Based on the WordPress Widgets API example:  http://codex.wordpress.org/Widgets_API
 */
class DailymileWidget extends WP_Widget {
	/** constructor */
	function DailymileWidget() {
		parent::WP_Widget(false, $name = 'dailymile', array('description' => "Faster, cached Dailymile widgets."));
		wp_enqueue_style($this->widget_options['classname'], 'http://www.dailymile.com/stylesheets/widgets/training.css', array(), false, 'all');
		
	}

	function get_dm_widget($widget_type, $username) {
		$widget_sources = array(
			'last_workout'		=> 'http://www.dailymile.com/people/' . $username . '/training/widget.js',
			'distance_large'	=> 'http://www.dailymile.com/people/' . $username . '/widgets/distance/large.js',
			'distance_mini'		=> 'http://www.dailymile.com/people/' . $username . '/widgets/distance/mini.js',
			'events'			=> 'http://www.dailymile.com/people/' . $username . '/events/widget.js',
			'donuts'			=> 'http://www.dailymile.com/people/' . $username . '/widgets/food/donuts.js',
			'cheeseburgers'		=> 'http://www.dailymile.com/people/' . $username . '/widgets/food/burgers.js'
		);
		if (false === ( $widget_source = get_transient($this->id) ) ) {
		    $widget_source = wp_remote_get($widget_sources[$widget_type]); 
			if ($widget_type == 'events') $this->parse_dm_races_widget($widget_source['body']);
			$widget_source['body'] = str_replace(array('(function() {', "document.write('", "');", '})();', 'var now = new Date();' ), '', $widget_source['body']);
			$widget_source['body'] = str_replace(array('\n', "}\n"), "\n", $widget_source['body']);
			$widget_source['body'] = stripslashes($widget_source['body']);
			$widget_source['body'] = preg_replace('%^.*stylesheets/widgets/training.css.*%', '', $widget_source['body']);
			$widget_source['body'] = preg_replace('/if\\(new Date\\(\d{4},\d{2} - 1,(\d{2},?){4}\\) > now\\) \\{/', '', $widget_source['body']);
			set_transient($this->id, $widget_source, 30 * 60 * 60); // update cached widget source every 30 minutes
		}
		return $widget_source['body'];
	}
	
	
	function parse_dm_races_widget($src) {
		$output = array();
		$lines = explode("\n", $src);
		foreach ($lines as $line){
			if (strpos($line, 'if(new Date(')) {
					// dailymile races widget dates look like this: 2010,12 - 1,05,08,00,00
					$datestring =  substr($line, strrpos($line, 'new Date(') + 9, 23);
					list($year, $month, $day, $hour, $minutes, $seconds) = explode(',', $datestring);
					if (mktime($hour,$minute,$second,substr($month, 0, 1),$day,$year) > time())
						$output[] = $line;
			} else {
				$output[] = $line;
			}
		}
		return implode("\n", $output);
	}
	

	function widget($args, $instance) {
		extract( $args );
		if (empty($instance['user'])) return;
			$title = apply_filters('widget_title', $instance['title']);
			?>
				<?php echo $before_widget; ?>
					<?php if ( $title )
						echo $before_title . $title . $after_title; ?>
					<?php echo $this->get_dm_widget($instance['widget'], $instance['user']); ?>
				<?php echo $after_widget; ?>
		<?php
	}

	function update($new_instance, $old_instance) {				
		$instance = $old_instance;
		$instance['title'] = strip_tags($new_instance['title']);
		$instance['user'] = $new_instance['user'];
		$instance['widget'] = $new_instance['widget'];
		delete_transient($this->id);
		return $instance;
	}

	function form($instance) {
		$widget_types = array(	'Last Workout' =>'last_workout', 
								'Total Distance' => 'distance_large', 
								'Mini Mileage Counter' => 'distance_mini', 
								'Racing Widget' => 'events', 
								'Donuts' => 'donuts', 
								'Cheezburgers' => 'cheeseburgers');
		
		$title = esc_attr($instance['title']);
		$user = esc_attr($instance['user']);
		$widget = esc_attr($instance['widget']);
		$user_required = (empty($user)) ? '<span style="color:#c43; font-weight: bold"> Required</span>' : '';
		?>
			<p><label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:'); ?> <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo $title; ?>" /></label></p>
			<p><label for="<?php echo $this->get_field_id('user'); ?>">Dailymile <?php _e('Username:'); echo $user_required; ?> <input class="widefat" id="<?php echo $this->get_field_id('user'); ?>" name="<?php echo $this->get_field_name('user'); ?>" type="text" value="<?php echo $user; ?>" /></label></p>
			<p><label for="<?php echo $this->get_field_id('widget'); ?>"><?php _e('Widgets:'); ?> <select id="<?php echo $this->get_field_id('widget'); ?>" name="<?php echo $this->get_field_name('widget'); ?>" size="1">
			<?php
				foreach($widget_types as $label => $val):
					$selected = ($widget == $val) ? ' selected="selected"' : '';
					echo "<option value='$val' $selected>$label</option>\n";
				endforeach;
			?>
			</select></label></p>
		<?php 
	}
}

add_action('widgets_init', create_function('', 'return register_widget("DailymileWidget");'));








?>